#include "DXUT.h"
#include "DxaDisplacement.h"

Dxa::Displacement::Displacement(ID3D11Device* device)
	:Dxa::Sas(device)
{
	#pragma region 6.4
	camera.SetViewParams( &D3DXVECTOR3(0, 3, -10), &D3DXVECTOR3(0, 3, 0) );
	#pragma endregion set up camera view
}

void Dxa::Displacement::loadEffect()
{
	#pragma region 6.5
	ID3DBlob* compiledEffect = NULL;
	ID3DBlob* compilationErrors = NULL;
	HRESULT hr = D3DX11CompileFromFileW(
			L"fx/displacement.fx", NULL, NULL, NULL,
			"fx_5_0", 0, 0, NULL, &compiledEffect, &compilationErrors, NULL);
	if(hr != S_OK)
	{
		if(compilationErrors != NULL)
			MessageBoxA( NULL, 
				(LPSTR)compilationErrors->GetBufferPointer(),
				"Failed to create effect from file!", MB_OK);
		else
			MessageBoxA( NULL, 
				"File cound not be opened",
				"Failed to create effect from file!", MB_OK);
		exit(-1);
	}
	#pragma endregion compile effect file with error handling
	#pragma region 6.6
	hr = D3DX11CreateEffectFromMemory(
		compiledEffect->GetBufferPointer(), 
		compiledEffect->GetBufferSize(),
		0, device, &effect);
	if(hr != S_OK)
	{
		MessageBoxA( NULL, 
			"CreateEffectFromMemory failed",
			"Failed to create effect from file!", MB_OK);
		exit(-1);
	}
	#pragma endregion create effect with error handling
}

HRESULT Dxa::Displacement::createResources()
{
	Dxa::Sas::createResources();

	inputLayoutManager = new InputLayoutManager();

	#pragma region 6.7
	mesh = NULL;
	Mesh::createMeshFromFile("media/shape.dgb", device, &mesh);
	if(mesh == NULL)
	{
		MessageBoxA( NULL, 
			"Mesh could not be loaded.",
			"Failed to create mesh from file!", MB_OK);
		exit(-1);
	}
	#pragma endregion create mesh from dgb file

	#pragma region 8.5
	D3DX11_PASS_DESC basicPassDesc;
	effect->GetTechniqueByName("normalmapped")->GetPassByName("normalmapped")->GetDesc(&basicPassDesc);
	inputLayout = inputLayoutManager->getCompatibleInputLayout(device, basicPassDesc, mesh, false);
	#pragma endregion create input layout for rendering the mesh with the textured technique

	#pragma region 8.3
	D3DX11CreateShaderResourceViewFromFile( device, L"Media/5902-diffuse.jpg", NULL, NULL, &kdTextureSrv, NULL);
	D3DX11CreateShaderResourceViewFromFile( device, L"Media/5902-bump.jpg", NULL, NULL, &bumpTextureSrv, NULL);
	D3DX11CreateShaderResourceViewFromFile( device, L"Media/5902-normal.jpg", NULL, NULL, &normalTextureSrv, NULL);
	#pragma endregion load texture, create shader resource view

	#pragma region 10.1
	D3DX11CreateShaderResourceViewFromFile( device, L"Media/cloudyNoon.dds", NULL, NULL, &envTextureSrv, NULL);
	#pragma endregion load environment cube texture, create shader resource view

	#pragma region 12.4
	Mesh::createQuad(device, &backgroundQuad);
	D3DX11_PASS_DESC backgroundPassDesc;
	effect->GetTechniqueByName("envmapped")->GetPassByName("background")->GetDesc(&backgroundPassDesc);
	backgroundInputLayout = inputLayoutManager->getCompatibleInputLayout(device, backgroundPassDesc, backgroundQuad, false);

	#pragma endregion background quad mesh and input layout for the background technique pass

	return S_OK;
}

HRESULT Dxa::Displacement::releaseResources()
{
	#pragma region 12.5
	delete backgroundQuad;
	#pragma endregion release background quad mesh and input layout

	#pragma region 10.1
	envTextureSrv->Release();
	#pragma endregion release environment cube texture srv
	#pragma region 8.4
	kdTextureSrv->Release();
	bumpTextureSrv->Release();
	normalTextureSrv->Release();
	#pragma endregion release texture srv
	#pragma region 6.9
	delete mesh;
	#pragma endregion release resources

	delete inputLayoutManager;

	return Dxa::Sas::releaseResources();
}

HRESULT Dxa::Displacement::createSwapChainResources()
{
	Dxa::Sas::createSwapChainResources();

	#pragma region 6.10
	camera.SetProjParams(1.58, (float)backbufferSurfaceDesc.Width / backbufferSurfaceDesc.Height, 0.1, 1000 );
	#pragma endregion set camera aspect ratio
	return S_OK;
}

HRESULT Dxa::Displacement::releaseSwapChainResources()
{
	#pragma region 6.11
	#pragma endregion do nothing
	
	return Dxa::Sas::releaseSwapChainResources();
}

#pragma region 6.12
void Dxa::Displacement::animate(double dt, double t)
{
	Dxa::Sas::animate(dt, t);
	camera.FrameMove(dt);
}

bool Dxa::Displacement::processMessage( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	if(Dxa::Sas::processMessage(hWnd, uMsg, wParam, lParam))
		return true;
	return camera.HandleMessages(hWnd, uMsg, wParam, lParam);
}
#pragma endregion pass messages to camera object

void Dxa::Displacement::render(ID3D11DeviceContext* context)
{
	#pragma region 6.13
	float clearColor[4] = { 0.3f, 0.2f, 0.9f, 0.0f };

	ID3D11RenderTargetView* defaultRtv = DXUTGetD3D11RenderTargetView();
	ID3D11DepthStencilView* defaultDsv = DXUTGetD3D11DepthStencilView();
	context->ClearRenderTargetView( defaultRtv, clearColor );
	context->ClearDepthStencilView( defaultDsv, D3D11_CLEAR_DEPTH, 1.0, 0 );
	#pragma endregion clear backbuffer and depth

	#pragma region 6.14
	D3DXMATRIX modelMatrix, modelMatrixInverse;
	D3DXMatrixIdentity(&modelMatrix);
	D3DXMatrixInverse(&modelMatrixInverse, NULL, &modelMatrix);
	D3DXMATRIX viewMatrix = *camera.GetViewMatrix();
	D3DXMATRIX projMatrix = *camera.GetProjMatrix();
	D3DXMATRIX modelViewProjMatrix = modelMatrix * viewMatrix * projMatrix;
	#pragma endregion compute matrices

	#pragma region 6.15
	effect->GetVariableByName("modelMatrix")->AsMatrix()->SetMatrix((float*)&modelMatrix);
	effect->GetVariableByName("modelMatrixInverse")->AsMatrix()->SetMatrix((float*)&modelMatrixInverse);
	effect->GetVariableByName("modelViewProjMatrix")->AsMatrix()->SetMatrix((float*)&modelViewProjMatrix);
	#pragma endregion set global effect variables

	#pragma region 12.6
	D3DXMATRIX eyeMatrix;
	D3DXMatrixTranslation(&eyeMatrix, camera.GetEyePt()->x, camera.GetEyePt()->y, camera.GetEyePt()->z);
	D3DXMATRIX eyeViewProjMatrix = eyeMatrix * viewMatrix * projMatrix;
	D3DXMATRIX eyeViewProjMatrixInverse;
	D3DXMatrixInverse(&eyeViewProjMatrixInverse, NULL, &eyeViewProjMatrix);
	effect->GetVariableByName("eyeViewProjMatrixInverse")->AsMatrix()->SetMatrix((float*)&eyeViewProjMatrixInverse);
	#pragma endregion compute screen-to-viewdir matrix

	#pragma region 10.2
	effect->GetVariableByName("eyePos")->AsVector()->SetFloatVector((float*)camera.GetEyePt());
	effect->GetVariableByName("kdTexture")->AsShaderResource()->SetResource(kdTextureSrv);
	effect->GetVariableByName("bumpTexture")->AsShaderResource()->SetResource(bumpTextureSrv);
	effect->GetVariableByName("normalTexture")->AsShaderResource()->SetResource(normalTextureSrv);
	effect->GetVariableByName("envTexture")->AsShaderResource()->SetResource(envTextureSrv);
	context->IASetInputLayout(inputLayout);
	effect->GetTechniqueByName("normalmapped")->GetPassByName("normalmapped")->Apply(0, context);
	mesh->draw(context);
	#pragma endregion draw mesh using technique basic

	#pragma region 12.7
	context->IASetInputLayout(backgroundInputLayout);
	effect->GetTechniqueByName("envmapped")->GetPassByName("background")->Apply(0, context);
	backgroundQuad->draw(context);
	#pragma endregion draw background

	Dxa::Sas::render(context);
}

