#include "DXUT.h"
#include "DxaTextured.h"


Dxa::Textured::Textured(ID3D11Device* device)
	:Dxa::Base(device)
{
	#pragma region 6.4
	camera.SetViewParams( &D3DXVECTOR3(0, 3, -10), &D3DXVECTOR3(0, 3, 0) );
	#pragma endregion set up camera view
}


HRESULT Dxa::Textured::createResources()
{
	#pragma region 6.5
	ID3DBlob* compiledEffect = NULL;
	ID3DBlob* compilationErrors = NULL;
	HRESULT hr = D3DX11CompileFromFileW(
			L"fx/textured.fx", NULL, NULL, NULL,
			"fx_5_0", 0, 0, NULL, &compiledEffect, &compilationErrors, NULL);
	if(hr != S_OK)
	{
		if(compilationErrors != NULL)
			MessageBoxA( NULL, 
				(LPSTR)compilationErrors->GetBufferPointer(),
				"Failed to create effect from file!", MB_OK);
		else
			MessageBoxA( NULL, 
				"File cound not be opened",
				"Failed to create effect from file!", MB_OK);
		exit(-1);
	}
	#pragma endregion compile effect file with error handling
	#pragma region 6.6
	hr = D3DX11CreateEffectFromMemory(
		compiledEffect->GetBufferPointer(), 
		compiledEffect->GetBufferSize(),
		0, device, &effect);
	if(hr != S_OK)
	{
		MessageBoxA( NULL, 
			"CreateEffectFromMemory failed",
			"Failed to create effect from file!", MB_OK);
		exit(-1);
	}
	#pragma endregion create effect with error handling

	#pragma region 6.7
	mesh = NULL;
	Mesh::createMeshFromFile("media/bigbunny.dgb", device, &mesh);
	if(mesh == NULL)
	{
		MessageBoxA( NULL, 
			"Mesh could not be loaded.",
			"Failed to create mesh from file!", MB_OK);
		exit(-1);
	}
	#pragma endregion create mesh from dgb file

	#pragma region 8.5
	D3DX11_PASS_DESC basicPassDesc;
	effect->GetTechniqueByName("textured")->GetPassByName("textured")->GetDesc(&basicPassDesc);
	mesh->createInputLayout(device, &basicPassDesc, &inputLayout);
	#pragma endregion create input layout for rendering the mesh with the textured technique


	#pragma region 8.3
	D3DX11CreateShaderResourceViewFromFile( device, L"Media/uvGrid.jpg", NULL, NULL, &kdTextureSrv, NULL);
	#pragma endregion load texture, create shader resource view

	return S_OK;
}

HRESULT Dxa::Textured::releaseResources()
{
	#pragma region 8.4
	kdTextureSrv->Release();
	#pragma endregion release texture srv
	#pragma region 6.9
	effect->Release();
	inputLayout->Release();
	delete mesh;
	#pragma endregion release resources
	return S_OK;
}

HRESULT Dxa::Textured::createSwapChainResources()
{
	#pragma region 6.10
	camera.SetProjParams(1.58, (float)backbufferSurfaceDesc.Width / backbufferSurfaceDesc.Height, 0.1, 1000 );
	#pragma endregion set camera aspect ratio
	return S_OK;
}

HRESULT Dxa::Textured::releaseSwapChainResources()
{
	#pragma region 6.11
	#pragma endregion do nothing
	return S_OK;
}

#pragma region 6.12
void Dxa::Textured::animate(double dt, double t)
{
	camera.FrameMove(dt);
}
bool Dxa::Textured::processMessage( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	return camera.HandleMessages(hWnd, uMsg, wParam, lParam);
}
#pragma endregion pass messages to camera object

void Dxa::Textured::render(ID3D11DeviceContext* context)
{
	#pragma region 6.13
	float clearColor[4] = { 0.3f, 0.2f, 0.9f, 0.0f };

	ID3D11RenderTargetView* defaultRtv = DXUTGetD3D11RenderTargetView();
	ID3D11DepthStencilView* defaultDsv = DXUTGetD3D11DepthStencilView();
	context->ClearRenderTargetView( defaultRtv, clearColor );
	context->ClearDepthStencilView( defaultDsv, D3D11_CLEAR_DEPTH, 1.0, 0 );
	#pragma endregion clear backbuffer and depth

	#pragma region 6.14
	D3DXMATRIX modelMatrix, modelMatrixInverse;
	D3DXMatrixIdentity(&modelMatrix);
	D3DXMatrixInverse(&modelMatrixInverse, NULL, &modelMatrix);
	D3DXMATRIX viewMatrix = *camera.GetViewMatrix();
	D3DXMATRIX projMatrix = *camera.GetProjMatrix();
	D3DXMATRIX modelViewProjMatrix = modelMatrix * viewMatrix * projMatrix;
	#pragma endregion compute matrices

	#pragma region 6.15
	effect->GetVariableByName("modelMatrix")->AsMatrix()->SetMatrix((float*)&modelMatrix);
	effect->GetVariableByName("modelMatrixInverse")->AsMatrix()->SetMatrix((float*)&modelMatrixInverse);
	effect->GetVariableByName("modelViewProjMatrix")->AsMatrix()->SetMatrix((float*)&modelViewProjMatrix);
	#pragma endregion set global effect variables

	#pragma region 8.6
	effect->GetVariableByName("kdTexture")->AsShaderResource()->SetResource(kdTextureSrv);
	context->IASetInputLayout(inputLayout);
	effect->GetTechniqueByName("textured")->GetPassByName("textured")->Apply(0, context);
	mesh->draw(context);
	#pragma endregion draw mesh using technique basic

}
