#include "DXUT.h"
#include "DxaTriangleWithFx.h"

#pragma region 2.1
Dxa::TriangleWithFx::TriangleWithFx(ID3D11Device* device)
	:Base(device)
{
}

HRESULT Dxa::TriangleWithFx::createResources()
{
	#pragma region 4.7
	ID3DBlob* compiledEffect = NULL;
	D3DX11CompileFromFileW(
			L"fx/idle.fx", NULL, NULL, NULL,
			"fx_5_0", 0, 0, NULL, &compiledEffect, NULL, NULL);
	D3DX11CreateEffectFromMemory(
		compiledEffect->GetBufferPointer(), 
		compiledEffect->GetBufferSize(),
		0, device, &effect);
	#pragma endregion effect creation from .fx file

	#pragma region 2.6
	D3D11_BUFFER_DESC desc;
	desc.BindFlags = D3D11_BIND_VERTEX_BUFFER;
	desc.ByteWidth = sizeof(D3DXVECTOR3) * 3;
	desc.CPUAccessFlags = 0;
	desc.MiscFlags = 0;
	desc.StructureByteStride = sizeof(D3DXVECTOR3);
	desc.Usage = D3D11_USAGE_IMMUTABLE;
	D3DXVECTOR3 vertexPositionArray[3] = { D3DXVECTOR3(0, 0, 0.5), D3DXVECTOR3(0, 1, 0.5), D3DXVECTOR3(1, 0, 0.5) };
	D3D11_SUBRESOURCE_DATA initData;
	initData.pSysMem = vertexPositionArray;
	initData.SysMemPitch = 0;
	initData.SysMemSlicePitch = 0;
	device->CreateBuffer(&desc, &initData, &vertexBuffer);
	#pragma endregion create vertex buffer

	#pragma region 4.4
	//#pragma region 2.7
	//const char* vertexShaderCode = "float4 vsIdle(float4 pos : POSITION) : SV_Position {return pos;}";
	//ID3DBlob* vertexShaderByteCode;
	//D3DX11CompileFromMemory(vertexShaderCode, strlen(vertexShaderCode), NULL, NULL, NULL, "vsIdle", "vs_5_0", 0, 0, NULL, 
	//	&vertexShaderByteCode, NULL, NULL);
	//device->CreateVertexShader(vertexShaderByteCode->GetBufferPointer(), vertexShaderByteCode->GetBufferSize(), NULL, &vertexShader);
	//#pragma endregion create idle vertex shader
	#pragma endregion deleted code for vertex shader creation

	#pragma region 2.8
	D3D11_INPUT_ELEMENT_DESC positionElement;
	positionElement.AlignedByteOffset = 0;
	positionElement.Format = DXGI_FORMAT_R32G32B32_FLOAT;
	positionElement.InputSlot = 0;
	positionElement.InputSlotClass = D3D11_INPUT_PER_VERTEX_DATA;
	positionElement.InstanceDataStepRate = 0;
	positionElement.SemanticIndex = 0;
	positionElement.SemanticName = "POSITION";
	#pragma endregion vertex structure description for a vertex structure of a single float3 position

	#pragma region 4.5
	//#pragma region 2.9
	//device->CreateInputLayout(&positionElement, 1, vertexShaderByteCode->GetBufferPointer(), vertexShaderByteCode->GetBufferSize(), &inputLayout); 
	//#pragma endregion create an input layout mapping vertex structure element to vertex shader input parameter
	#pragma endregion deleted code for input layout creation

	#pragma region 4.6
	//#pragma region 2.10
	//const char* pixelShaderCode = "float4 psIdle() : SV_Target {return float4(1, 0, 0, 1);}";
	//ID3DBlob* pixelShaderByteCode;
	//D3DX11CompileFromMemory(pixelShaderCode, strlen(pixelShaderCode), NULL, NULL, NULL, "psIdle", "ps_5_0", 0, 0, NULL, 
	//	&pixelShaderByteCode, NULL, NULL);
	//device->CreatePixelShader(pixelShaderByteCode->GetBufferPointer(), pixelShaderByteCode->GetBufferSize(), NULL, &pixelShader);
	//#pragma endregion create idle pixel shader
	#pragma endregion deleted code for pixel shader creation

	#pragma region 4.8
	D3DX11_PASS_DESC idlePassDesc;
	effect->GetTechniqueByName("idle")->GetPassByName("idle")->GetDesc(&idlePassDesc);
	device->CreateInputLayout(&positionElement, 1, idlePassDesc.pIAInputSignature, idlePassDesc.IAInputSignatureSize, &inputLayout); 
	#pragma endregion input layout creation for shader defined in .fx file

	return S_OK;
}

HRESULT Dxa::TriangleWithFx::releaseResources()
{
	#pragma region 4.9
	effect->Release();
	#pragma endregion release effect
	#pragma region 2.11
	vertexBuffer->Release();
	inputLayout->Release();
	#pragma endregion release all resources created in method createResources
	return S_OK;
}

void Dxa::TriangleWithFx::render(ID3D11DeviceContext* context)
{
	#pragma region 2.12
	float clearColor[4] = { 0.9f, 0.7f, 0.1f, 0.0f };

	ID3D11RenderTargetView* defaultRtv = DXUTGetD3D11RenderTargetView();
	ID3D11DepthStencilView* defaultDsv = DXUTGetD3D11DepthStencilView();
	context->ClearRenderTargetView( defaultRtv, clearColor );
	context->ClearDepthStencilView( defaultDsv, D3D11_CLEAR_DEPTH, 1.0, 0 );
	#pragma endregion clear backbuffer and depth

	#pragma region 2.13
	unsigned int stride = sizeof(D3DXVECTOR3);
	unsigned int offset = 0;
	context->IASetVertexBuffers(0, 1, &vertexBuffer, &stride, &offset);
	context->IASetPrimitiveTopology( D3D11_PRIMITIVE_TOPOLOGY_TRIANGLELIST );
	context->IASetInputLayout(inputLayout);
	
	#pragma region 4.11
	effect->GetTechniqueByName("idle")->GetPassByName("idle")->Apply(0, context);
	#pragma endregion set up shaders from effect

	context->Draw(3, 0);
	#pragma endregion draw triangle

}

#pragma endregion Dxa::TriangleWithFx method definitions