#include "DXUT.h"
#include "InputLayoutManager.h"
#include "Mesh.h"

InputLayoutManager::InputLayoutManager()
{
}

InputLayoutManager::~InputLayoutManager()
{
	InputConfigurationList::iterator i = inputConfigurationList.begin();
	InputConfigurationList::iterator e = inputConfigurationList.end();
	while(i != e)
	{
		i->inputLayout->Release();
		i++;
	}
}

InputLayoutManager::InputConfiguration::InputConfiguration(const D3DX11_PASS_DESC& passDesc, Mesh* mesh, bool instancing)
	:passDesc(passDesc)
{
	mesh->getElements(&elements, &nElements, instancing);
}

HRESULT InputLayoutManager::InputConfiguration::createInputLayout(ID3D11Device* device)
{
	return device->CreateInputLayout(elements, nElements, passDesc.pIAInputSignature, passDesc.IAInputSignatureSize, &inputLayout);
}

bool InputLayoutManager::InputConfiguration::isCompatible(const InputLayoutManager::InputConfiguration& other) const
{
	if(passDesc.IAInputSignatureSize != other.passDesc.IAInputSignatureSize)
		return false;
	if(0 != memcmp((const void*)passDesc.pIAInputSignature, (const void*)other.passDesc.pIAInputSignature, passDesc.IAInputSignatureSize))
		return false;
	for(unsigned int i=0; i<nElements; i++)
	{
		for(unsigned int u=0; u<other.nElements; u++)
		{
			if(elements[i].SemanticIndex == other.elements[u].SemanticIndex &&
				(0 == strcmp(elements[i].SemanticName, other.elements[u].SemanticName)))
			{
				if(elements[i].AlignedByteOffset != other.elements[u].AlignedByteOffset)
					return false;
				if(elements[i].Format != other.elements[u].Format)
					return false;
				if(elements[i].InputSlot != other.elements[u].InputSlot)
					return false;
				if(elements[i].InputSlotClass != other.elements[u].InputSlotClass)
					return false;
				if(elements[i].InstanceDataStepRate != other.elements[u].InstanceDataStepRate)
					return false;
			}
		}
	}
	return true;
}

ID3D11InputLayout* InputLayoutManager::getCompatibleInputLayout(ID3D11Device* device, const D3DX11_PASS_DESC& passDesc, Mesh* mesh, bool instancing)
{
	InputConfiguration tentativeInputConfiguration(passDesc, mesh, instancing);
	InputConfigurationList::iterator i = inputConfigurationList.begin();
	InputConfigurationList::iterator e = inputConfigurationList.end();
#ifdef DEBUG
	if(tentativeInputConfiguration.createInputLayout(device) != S_OK)
	{
		MessageBox(NULL, L"Failed to create an input layout. Invalid mesh-technique combination.", L"Error!", 0);
		DXUTShutdown(-1);
	}
	while(i != e)
	{
		if(i->isCompatible(tentativeInputConfiguration))
		{
			tentativeInputConfiguration.inputLayout->Release();
			return i->inputLayout;
		}
		i++;
	}
#else
	while(i != e)
	{
		if(i->isCompatible(tentativeInputConfiguration))
			return i->inputLayout;
		i++;
	}
	tentativeInputConfiguration.createInputLayout(device);
#endif
	inputConfigurationList.push_back(tentativeInputConfiguration);
	return tentativeInputConfiguration.inputLayout;
}