#pragma once
#include "d3dx11effect.h"

struct aiMesh;

class Mesh
{
	ID3D11Buffer* vertexBuffer;
	ID3D11Buffer* indexBuffer;
	D3D11_INPUT_ELEMENT_DESC* elements;
	unsigned int nElements;
	unsigned int nElementsInstancing;

	unsigned int vertexStride;
	bool wideIndexBuffer;

	unsigned int nVertices;
	unsigned int nFaces;

	D3D11_PRIMITIVE_TOPOLOGY topology;

	HRESULT create(
		ID3D11Device* device,
		D3D11_INPUT_ELEMENT_DESC* elements,
		unsigned int nElements,
		unsigned int nVertices,
		void* vertexData,
		unsigned int nFaces,
		void* indexData,
		bool wideIndexBuffer);
	Mesh();
	static HRESULT createMeshFromMemory(ID3D11Device* device, BYTE* data, unsigned int nBytes, Mesh** mesh);
public:
	static HRESULT createMeshFromFile(const char* dgbFilename, ID3D11Device* device, Mesh** mesh);
	static HRESULT createQuad(ID3D11Device* device, Mesh** mesh);
	static HRESULT importMesh(aiMesh* assMesh, ID3D11Device* device, Mesh** mesh, const void** vertexData=NULL, const void** indexData=NULL);
	~Mesh();
	HRESULT import(
		ID3D11Device* device,
		aiMesh* assMesh,
		const void** vertexData=NULL,
		const void** indexData=NULL);

	HRESULT createInputLayout(ID3D11Device* device, D3DX11_PASS_DESC* passDesc, ID3D11InputLayout** inputLayout);

	void getElements(const D3D11_INPUT_ELEMENT_DESC** elements, unsigned int* nElements, bool instancing=false);

	void draw(ID3D11DeviceContext* context);

	unsigned int getVertexCount() {return nVertices;}
	unsigned int getFaceCount() {return nFaces;}
	unsigned int getVertexStride() {return vertexStride;}
	unsigned int getIndexStride() {return wideIndexBuffer?sizeof(unsigned int):sizeof(unsigned short);}
};

